@@ -42,8 +42,20 @@ from oslo_concurrency._i18n import _
 # time module as the check because that's a monkey patched module we use
 # in combination with subprocess below, so they need to match.
 eventlet = importutils.try_import('eventlet')
-if eventlet and eventlet.patcher.is_monkey_patched(time):
-    from eventlet.green import subprocess
+eventlet_patched = eventlet and eventlet.patcher.is_monkey_patched(time)
+if eventlet_patched:
+    if os.name == 'nt':
+        # subprocess.Popen.communicate will spawn two threads consuming
+        # stdout/stderr when passing data through stdin. We need to make
+        # sure that *native* threads will be used as pipes are blocking
+        # on Windows.
+        # Recent eventlet versions actually do patch subprocess.
+        subprocess = eventlet.patcher.original('subprocess')
+        subprocess.threading = eventlet.patcher.original('threading')
+    else:
+        from eventlet.green import subprocess
+
+    from eventlet import tpool
 else:
     import subprocess
 
@@ -250,6 +262,10 @@ def execute(*cmd, **kwargs):
     :param prlimit:         Set resource limits on the child process. See
                             below for a detailed description.
     :type prlimit:          :class:`ProcessLimits`
+    :param python_exec:     The python executable to use for enforcing
+                            prlimits. If this is not set it will default to use
+                            sys.executable.
+    :type python_exec:      string
     :returns:               (stdout, stderr) from process execution
     :raises:                :class:`UnknownArgumentError` on
                             receiving unknown arguments
@@ -302,6 +318,7 @@ def execute(*cmd, **kwargs):
     on_completion = kwargs.pop('on_completion', None)
     preexec_fn = kwargs.pop('preexec_fn', None)
     prlimit = kwargs.pop('prlimit', None)
+    python_exec = kwargs.pop('python_exec', sys.executable)
 
     if isinstance(check_exit_code, bool):
         ignore_exit_code = not check_exit_code
@@ -338,7 +355,7 @@ def execute(*cmd, **kwargs):
                     _('Process resource limits are ignored as '
                       'this feature is not supported on Windows.'))
         else:
-            args = [sys.executable, '-m', 'oslo_concurrency.prlimit']
+            args = [python_exec, '-m', 'oslo_concurrency.prlimit']
             args.extend(prlimit.prlimit_args())
             args.append('--')
             args.extend(cmd)
@@ -377,7 +394,14 @@ def execute(*cmd, **kwargs):
                 on_execute(obj)
 
             try:
-                result = obj.communicate(process_input)
+                # eventlet.green.subprocess is not really greenthread friendly
+                # on Windows. In order to avoid blocking other greenthreads,
+                # we have to wrap this call using tpool.
+                if eventlet_patched and os.name == 'nt':
+                    result = tpool.execute(obj.communicate,
+                                           process_input)
+                else:
+                    result = obj.communicate(process_input)
 
                 obj.stdin.close()  # pylint: disable=E1101
                 _returncode = obj.returncode  # pylint: disable=E1101
@@ -455,8 +479,10 @@ def trycmd(*args, **kwargs):
     the command's stdout and stderr.  If 'err' is not empty then the
     command can be considered to have failed.
 
-    :discard_warnings   True | False. Defaults to False. If set to True,
-                        then for succeeding commands, stderr is cleared
+    :param discard_warnings:  True | False. Defaults to False. If set to True,
+                              then for succeeding commands, stderr is cleared
+    :type discard_warnings:   boolean
+    :returns:                 (out, err) from process execution
 
     """
     discard_warnings = kwargs.pop('discard_warnings', False)
@@ -544,8 +570,8 @@ def ssh_execute(ssh, cmd, process_input=None,
 def get_worker_count():
     """Utility to get the default worker count.
 
-    @return: The number of CPUs if that can be determined, else a default
-             worker count of 1 is returned.
+    :returns: The number of CPUs if that can be determined, else a default
+              worker count of 1 is returned.
     """
     try:
         return multiprocessing.cpu_count()
